#!/usr/bin/perl -w
use strict;
use Event qw(loop);
use POSIX qw(EINPROGRESS);
use IO::Socket::INET;
use Socket qw(:all);

for my $port (@ARGV) {
    my $socket = IO::Socket::INET->new(proto => "tcp") || 
        die "Could not create socket: $!";
    $socket->blocking(0);
    if (connect($socket, pack_sockaddr_in($port, inet_aton("127.0.0.1")))) {
        activate($socket);
    } elsif ($! == EINPROGRESS) {
        my $watcher;
        $watcher = Event->io(fd => $socket,
                             poll => "w",
                             cb => sub { 
                                 $watcher->cancel;
                                 connect_result($socket);
                             });
    } else {
        print STDERR "Could not connect to 127.0.0.1:$port: $!";
    }
}

sub connect_result {
    my $socket = shift;
    my $packed = getsockopt($socket, SOL_SOCKET, SO_ERROR);
    if (my $rc = unpack("I", $packed)) {
        print STDERR "Could not connect: ", $! = $rc, "\n";
        return;
    }
    activate($socket);
}

sub activate {
    my $socket = shift;
    print STDERR "Do real work on $socket\n";
}

loop;
