#!/usr/bin/perl -w
use strict;
use IO::Socket::INET;
use POSIX qw(EINTR);

$SIG{PIPE} = "IGNORE";

defined(my $file = shift) || die "Usage: $0 file\n";

my $socket = IO::Socket::INET->new(PeerAddr => "127.0.0.1",
                                   PeerPort => 1234) || 
    die "Could not listen on 127.0.0.1:1234: $!";

my $socket_fd = fileno($socket);

my $read = 0;
my $reading = 1;
my $rd_mask = "";
vec($rd_mask, $socket_fd, 1) = 1;

open(my $fh, "<", $file) || die "Could not open $file: $!";
my $out_buffer = do { local $/; <$fh> };
my $writing;
my $wr_mask = "";
if ($out_buffer eq "") {
    $writing = 0;
    shutdown($socket, 1);
} else {
    $writing = 1;
    vec($wr_mask, $socket_fd, 1) = 1;
}

while ($reading || $writing) {
    my $nfound = 
        select(my $rd = $rd_mask, my $wr = $wr_mask, undef, undef);
    if ($nfound < 0) {
        next if $! == EINTR;
        die "Select error: $!";
    }
    if ($reading && vec($rd, $socket_fd, 1)) {
        my $rc = sysread($socket, my $buffer, 4096);
        if ($rc) {
            $read += $rc;
        } elsif (defined($rc)) {
            # EOF
            print STDERR "Read $read bytes\n";
            $reading = 0;
            $rd_mask = "";
            shutdown($socket, 0);
        } else {
            # Error
            die "Read error: $!" if $! != EINTR;
        }
    }
    if ($writing && vec($wr, $socket_fd, 1)) {
        if (defined(my $rc = syswrite($socket, $out_buffer))) {
            substr($out_buffer, 0, $rc, "");
            if ($out_buffer eq "") {
                $writing = 0;
                $wr_mask = "";
                shutdown($socket, 1);
            }
        } else {
            # Error
            die "Write error: $!" if $! != EINTR;
        }
    }
}
