#!/usr/bin/perl -w
use strict;
use POSIX qw(EINTR EWOULDBLOCK _exit);
use IO::Socket::INET;

# Modern UNIX:
$SIG{CHLD} = "IGNORE";
$SIG{PIPE} = "IGNORE";

defined(my $file = shift) || die "Usage: $0 file\n";

my $socket = IO::Socket::INET->new(LocalAddr => "127.0.0.1",
                                   LocalPort => 1234,
                                   Listen    => 5) ||
    die "Could not listen on 127.0.0.1:1234: $!\n";

while (1) {
    my $accepted = $socket->accept;
    # (my ($accepted, $peer_packed) = $socket->accept;
    if (!$accepted) {
        print STDERR "Error accepting connection: $!\n" unless 
            $! == EINTR;
        next;
    }
    defined(my $pid = fork) ||
        die "Could not fork: $!";

    if ($pid) {
        # Parent
        # Schrijf expliciet voor duidelijkheid
        close $accepted;
        next;
    }
    # Child
    close $socket;
    my $peer_port = $accepted->peerport;
    my $peer_ip   = $accepted->peerhost;
    print STDERR "Pid $$, received a connection from $peer_ip:$peer_port\n";

    $accepted->blocking(0);
    my $accepted_fd = fileno($accepted);

    my $read = 0;
    my $reading = 1;
    my $rd_mask = "";
    vec($rd_mask, $accepted_fd, 1) = 1;

    open(my $fh, "<", $file) || die "Could not open $file: $!";
    my $out_buffer = do { local $/; <$fh> };
    my $writing;
    my $wr_mask = "";
    if ($out_buffer eq "") {
        $writing = 0;
        shutdown($accepted, 1);
    } else {
        $writing = 1;
        vec($wr_mask, $accepted_fd, 1) = 1;
    }

    while ($reading || $writing) {
        my $nfound = 
            select(my $rd = $rd_mask, my $wr = $wr_mask, undef, undef);
        if ($nfound < 0) {
            next if $! == EINTR;
            die "Select error: $!";
        }
        if ($reading && vec($rd, $accepted_fd, 1)) {
            my $rc = sysread($accepted, my $buffer, 4096);
            if ($rc) {
                $read += $rc;
            } elsif (defined($rc)) {
                # EOF
                print STDERR "Read $read bytes\n";
                $reading = 0;
                $rd_mask = "";
                shutdown($accepted, 0);
            } else {
                # Error
                die "Read error: $!" unless $! == EINTR || $! == EWOULDBLOCK;
            }
        }
        if ($writing && vec($wr, $accepted_fd, 1)) {
            if (defined(my $rc = syswrite($accepted, $out_buffer))) {
                substr($out_buffer, 0, $rc, "");
                if ($out_buffer eq "") {
                    $writing = 0;
                    $wr_mask = "";
                    shutdown($accepted, 1);
                }
            } else {
                # Error
                die "Write error: $!" unless $! == EINTR || $! == EWOULDBLOCK;
            }
        }
    }
    _exit(0);
}
