#!/usr/bin/perl -w
use strict;
use Socket;
use POE qw(Wheel::SocketFactory Wheel::ReadWrite);

POE::Session->create
    (args => shift,
     inline_states => {
        _start		=> \&start,
        _default	=> \&default,

        accepted	=> \&accepted,
        listener_fails	=> \&listener_fails,

        want_num1	=> \&want_num1,
        want_num2	=> \&want_num2,
        connection_error=> \&connection_error,
    });
$poe_kernel->run();

sub default {
    print STDERR "calling non existant event $_[ARG0]\n" unless
        substr($_[ARG0], 0, 1) eq "_";
    return;
}

sub start {
    my ($heap, $port) = @_[HEAP, ARG0];
    $port ||= 1234;
    $heap->{listener} = POE::Wheel::SocketFactory->new
        (BindPort => $port,
         SuccessEvent => "accepted",
         FailureEvent => "listener_fails");
}

sub listener_fails {
    my ($heap, $operator, $errstr) = @_[HEAP, ARG0, ARG2];
    print STDERR "$operator: $errstr\n";
    delete $heap->{listener};
}

sub accepted {
    my ($heap, $socket, $peer_address, $peer_port) = @_[HEAP, ARG0..ARG2];
    printf(STDERR "Connection from %s:%d\n", 
           inet_ntoa($peer_address), $peer_port);
    my $wheel = POE::Wheel::ReadWrite->new
        (Handle     => $socket,
         InputEvent => "want_num1",
         ErrorEvent => "connection_error");
    $heap->{connections}{$wheel->ID} = {
        wheel => $wheel
    };
    $wheel->put("Number 1?");
}

sub connection_error {
    my ($heap, $operator, $errno, $errstr, $wheel_id) = @_[HEAP, ARG0..ARG3];
    my $connection = delete $heap->{connections}{$wheel_id};
    if ($errno) {
        print STDERR "$operator: $errstr\n";
    } else {
        # EOF
        print STDERR "Connection closed\n";
    }
}

sub want_num1 {
    my ($heap, $num1, $wheel_id) = @_[HEAP, ARG0, ARG1];
    my $connection = $heap->{connections}{$wheel_id};
    $connection->{num1} = $num1;
    $connection->{wheel}->event(InputEvent => "want_num2");
    $connection->{wheel}->put("Number 2?");
}

sub want_num2 {
    my ($heap, $num2, $wheel_id) = @_[HEAP, ARG0, ARG1];
    my $connection = $heap->{connections}{$wheel_id};
    my $num1 = $connection->{num1};
    $connection->{wheel}->put(sprintf("%d*%d=%d", $num1, $num2, $num1*$num2));
    $connection->{wheel}->event(InputEvent => "want_num1");
    $connection->{wheel}->put("Number 1?");
}
